package com.flyer.foster.service.impl;

import cn.dev33.satoken.session.SaSession;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.asymmetric.KeyType;
import cn.hutool.crypto.asymmetric.RSA;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.flyer.exception.BusinessException;
import com.flyer.foster.consts.LoginDevice;
import com.flyer.foster.consts.RedisKeyConst;
import com.flyer.foster.dto.LoginDTO;
import com.flyer.foster.dto.UserAddDto;
import com.flyer.foster.dto.UserQueryDTO;
import com.flyer.foster.dto.UserRespDTO;
import com.flyer.foster.entity.Role;
import com.flyer.foster.entity.User;
import com.flyer.foster.entity.UserRole;
import com.flyer.foster.mapper.IUserMapper;
import com.flyer.foster.service.IRoleService;
import com.flyer.foster.service.IUserRoleService;
import com.flyer.foster.service.IUserService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.flyer.util.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author flyer
 * @since 2024-05-06
 */
@Slf4j
@Service
public class UserServiceImpl extends ServiceImpl<IUserMapper, User> implements IUserService {
    @Value("${use-local-private-key:false}")
    private boolean useLocalPrivateKey;

    @Value("${rsa-public-key}")
    private String publicKey;

    @Value("${rsa-private-key}")
    private String privateKey;

    @Value("${spring.profiles.active}")
    private String env;

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private IRoleService iRoleService;

    @Autowired
    private IUserRoleService iUserRoleService;

    @Override
    public String getRsaKey(String username) {
        if (useLocalPrivateKey) {
            return publicKey;
        } else {
            RSA rsa = new RSA();
            // 私钥写入缓存，当登录成功之后则删除私钥缓存
            redisUtil.set(RedisKeyConst.RSA_PRIVATE_KEY + username, rsa.getPrivateKeyBase64());
            // 把公钥返回给前端
            return rsa.getPublicKeyBase64();
        }
    }

    @Override
    public String login(LoginDTO loginDto) {
        String password = this.decryptPassword(loginDto);
        User user = this.lambdaQuery().eq(User::getUsername, loginDto.getUsername())
                .eq(User::getPassword, SecureUtil.md5(password))
                .one();
        if (user == null) {
            throw new BusinessException("账号密码错误！");
        }
        StpUtil.login(user.getUsername(), LoginDevice.PC);
        SaSession tokenSession = StpUtil.getTokenSession();
        tokenSession.set("tenantId", user.getTenantId());
        return StpUtil.getTokenValue();
    }

    /**
     * 对加密的密码进行解密
     *
     * @param loginDTO 登录对象
     */
    private String decryptPassword(LoginDTO loginDTO) {
        RSA rsa;
        String pwd = loginDTO.getPassword();
        if ("dev".equals(env)) {
            rsa = new RSA(privateKey, null);
        } else {
            log.info("rsa:private_key:" + loginDTO.getUsername());
            String privateKey = redisUtil.get(RedisKeyConst.RSA_PRIVATE_KEY + loginDTO.getUsername()).toString();
            rsa = new RSA(privateKey, null);
        }
        return rsa.decryptStr(pwd, KeyType.PrivateKey);
    }

    @Override
    public IPage<UserRespDTO> getPageList(IPage<User> page, UserQueryDTO queryDTO) {
        int tenantId = StpUtil.getTokenSession().getInt("tenantId");
        queryDTO.setTenantId(tenantId);
        // 查询管理员角色的账号
        Role adminRole = iRoleService.lambdaQuery().eq(Role::getIsAdmin, 1).eq(Role::getTenantId, tenantId).one();
        // 获取管理员用户id
        UserRole userRole = iUserRoleService.lambdaQuery().eq(UserRole::getRoleId, adminRole.getId()).one();
        queryDTO.setAdminUserId(userRole.getUserId());
        IPage<UserRespDTO> pageResult = baseMapper.getPageList(page, queryDTO);
        // 获取单个视频对象的标签集合
        for (UserRespDTO record : pageResult.getRecords()) {
            List<Integer> roleIdList = iUserRoleService.lambdaQuery()
                    .eq(UserRole::getUserId, record.getId()).list()
                    .stream().map(UserRole::getRoleId).collect(Collectors.toList());
            if (roleIdList.size() != 0) {
                List<Role> roleList = iRoleService.listByIds(roleIdList);
                record.setRoles(roleList);
            }
        }
        return pageResult;
    }

    @Transactional
    @Override
    public boolean removeByIdList(List<Integer> idList) {
        int tenantId = StpUtil.getTokenSession().getInt("tenantId");
        // 删除用户
        baseMapper.deleteByIdList(idList, tenantId);
        // 删除用户的角色
        iUserRoleService.deleteByUserIdList(idList);
        return true;
    }

    @Transactional
    @Override
    public boolean addUser(UserAddDto addDto) {
        addDto.setUsername(addDto.getUsername().trim());
        int tenantId = StpUtil.getTokenSession().getInt("tenantId");
        User user;
        if (addDto.getRoleId() == null) {
            throw new BusinessException("请选择角色");
        }
        user = this.lambdaQuery()
                .eq(User::getUsername, addDto.getUsername())
                .eq(User::getTenantId, tenantId)
                .one();
        if (user != null) {
            throw new BusinessException("用户名已存在");
        }
        user = new User();
        BeanUtil.copyProperties(addDto, user);
        user.setTenantId(tenantId);
        // md5加密
        user.setPassword(SecureUtil.md5(user.getPassword()));
        // 1.添加用户
        this.save(user);

        UserRole userRole = new UserRole();
        userRole.setUserId(user.getId());
        userRole.setRoleId(addDto.getRoleId());
        userRole.setTenantId(tenantId);
        // 2.添加用户角色
        iUserRoleService.save(userRole);
        return true;
    }

    @Override
    public List<UserRespDTO> getList(UserQueryDTO queryDTO) {
        int tenantId = StpUtil.getTokenSession().getInt("tenantId");
        queryDTO.setTenantId(tenantId);
        // 查询管理员角色的账号
        Role adminRole = iRoleService.lambdaQuery().eq(Role::getIsAdmin, 1).eq(Role::getTenantId, tenantId).one();
        // 获取管理员用户id
        UserRole userRole = iUserRoleService.lambdaQuery().eq(UserRole::getRoleId, adminRole.getId()).one();
        queryDTO.setAdminUserId(userRole.getUserId());
        return baseMapper.getList(queryDTO);
    }
}
